
      SUBROUTINE CURV1 (N,X,Y,SLP1,SLPN,YP,TEMP,T2,SIGMA)
C
C
C
C DIMENSION OF           X(N),Y(N),YP(N),TEMP(N)
C ARGUMENTS
C
C LATEST REVISION        FEBRUARY 5, 1974
C                        NOV. 30, 1977, STELLINGWERF,RUTGERS
C
C PURPOSE                CURV1 DETERMINES THE PARAMETERS NECESSARY TO
C                        COMPUTE AN INTERPOLATORY SPLINE UNDER TENSION
C                        THROUGH A SEQUENCE OF FUNCTIONAL VALUES.  THE
C                        SLOPES AT THE TWO ENDS OF THE CURVE MAY BE
C                        SPECIFIED OR OMITTED.  FOR ACTUAL COMPUTATION
C                        OF POINTS ON THE CURVE IT IS NECESSARY TO CALL
C                        THE FUNCTION CURV2.
C
C ACCESS CARDS           *FORTRAN,S=ULIB,N=CURV
C                        *COSY
C
C USAGE                  CALL CURV1 (N,X,Y,SLP1,SLPN,YP,TEMP,SIGMA)
C
C ARGUMENTS
C
C ON INPUT               N
C                          IS THE NUMBER OF VALUES TO BE INTERPOLATED
C                          (N .GE. 2).
C
C                        X
C                          IS AN ARRAY OF THE N INCREASING ABSCISSAE OF
C                          THE FUNCTIONAL VALUES.
C
C                        Y
C                          IS AN ARRAY OF THE N ORDINATES OF THE VALUES,
C                          (I.E., Y(K) IS THE FUNCTIONAL VALUE
C                          CORRESPONDING TO X(K)).
C
C                        SLP1 AND SLPN
C                          CONTAIN THE DESIRED VALUES FOR THE FIRST
C                          DERIVATIVE OF THE CURVE AT X(1) AND X(N),
C                          RESPECTIVELY.  IF THE QUANTITY SIGMA IS
C                          NEGATIVE THESE VALUES WILL BE DETERMINED
C                          INTERNALLY AND THE USER NEED ONLY FURNISH
C                          PLACE-HOLDING PARAMETERS FOR SLP1 AND SLPN.
C                          SUCH PLACE-HOLDING (DUMMY) PARAMETERS WILL BE
C                          IGNORED BUT NOT DESTROYED.
C
C                        YP
C                          IS AN ARRAY OF LENGTH AT LEAST N.
C
C                        TEMP
C                          IS AN ARRAY OF LENGTH AT LEAST N WHICH IS
C                          USED FOR SCRATCH STORAGE.
C
C                        SIGMA
C                          CONTAINS THE TENSION FACTOR.  THIS IS
C                          NON-ZERO AND INDICATES THE CURVINESS DESIRED.
C                          IF ABS(SIGMA) IS NEARLY ZERO (E.G., .001) THE
C                          RESULTING CURVE IS APPROXIMATELY A CUBIC
C                          SPLINE.  IF ABS(SIGMA) IS LARGE (E.G., 50.)
C                          THE RESULTING CURVE IS NEARLY A POLYGONAL
C                          LINE.  THE SIGN OF SIGMA INDICATES WHETHER
C                          THE DERIVATIVE INFORMATION HAS BEEN INPUT OR
C                          NOT.  IF SIGMA IS NEGATIVE THE ENDPOINT
C                          DERIVATIVES WILL BE DETERMINED INTERNALLY.
C                          A STANDARD VALUE FOR SIGMA IS APPROXIMATELY
C                          1. IN ABSOLUTE VALUE.
C
C                        T2
C                          SCRATCH ARRAY, DIMENSION N, USED ONLY
C                          FOR PERIODIC BC CASE
C
C PERIODIC BOUNDARY CONDITIONS--AUTOMATICALLY USED IF Y(1)=Y(N)
C
C ON OUTPUT              YP
C                          CONTAINS VALUES PROPORTIONAL TO THE SECOND
C                          DERIVATIVE OF THE CURVE AT THE GIVEN NODES.
C
C                        N, X, Y, SLP1, SLPN AND SIGMA ARE UNALTERED.
C
C ENTRY POINTS           CURV1
C
C SPECIAL CONDITIONS     NONE
C
C COMMON BLOCKS          NONE
C
C I/O                    NONE
C
C PRECISION              SINGLE
C
C REQUIRED ULIB          NONE
C ROUTINES
C
C SPECIALIST             RUSSELL K. REW, NCAR, BOULDER, COLORADO  80302
C
C LANGUAGE               FORTRAN
C
C HISTORY                WRITTEN BY A. K. CLINE, MARCH 1972.
C
C
C
C
      INTEGER         N
      REAL            X(N)       ,Y(N)       ,SLP1       ,SLPN       ,
     1                YP(N)      ,TEMP(N)    ,SIGMA
      REAL  T2(N)
      LOGICAL  PER

      PER=.FALSE.
      IF(Y(N).EQ.Y(1)) PER=.TRUE.
      NN = N
      NM1 = NN-1
      NP1 = NN+1
      DELX1 = X(2)-X(1)
      DX1 = (Y(2)-Y(1))/DELX1
C
C DETERMINE SLOPES IF NECESSARY
C
      IF (SIGMA .LT. 0.) GO TO 105
      SLPP1 = SLP1
      SLPPN = SLPN
C
C DENORMALIZE TENSION FACTOR
C
  101 SIGMAP = ABS(SIGMA)*FLOAT(NN-1)/(X(NN)-X(1))
C

C SET UP RIGHT HAND SIDE AND TRIDIAGONAL SYSTEM FOR YP AND
C PERFORM FORWARD ELIMINATION
C
      DELS = SIGMAP*DELX1
      EXPS = EXP(DELS)
      SINHS = .5*(EXPS-1./EXPS)
      SINHIN = 1./(DELX1*SINHS)
      SPDIAG = SINHIN*(SINHS-DELS)
      DIAG1 = SINHIN*(DELS*.5*(EXPS+1./EXPS)-SINHS)
      DIAGIN = 1./DIAG1
      IF(PER) GO TO 203
         YP(1) = DIAGIN*(DX1-SLPP1)
         TEMP(1) = DIAGIN*SPDIAG
      GO TO 200
 203  CONTINUE
         DX11=DX1
         DIAG11=DIAG1
         A2=SPDIAG
         TEMP(1)=0.
         YP(1)=0.
         T2(1)=-1.
         P=-1.
         Q=0.
         R=0.
 200  CONTINUE
      IF (NN .EQ. 2) GO TO 103
      DO 102 I=2,NM1
         DELX2 = X(I+1)-X(I)
         DX2 = (Y(I+1)-Y(I))/DELX2
         DELS = SIGMAP*DELX2
         EXPS = EXP(DELS)
         SINHS = .5*(EXPS-1./EXPS)
         SINHIN = 1./(DELX2*SINHS)
         DIAG2 = SINHIN*(DELS*(.5*(EXPS+1./EXPS))-SINHS)
         DIAGIN = 1./(DIAG1+DIAG2-SPDIAG*TEMP(I-1))
         YP(I) = DIAGIN*(DX2-DX1-SPDIAG*YP(I-1))
         IF(PER) T2(I)=-DIAGIN*SPDIAG*T2(I-1)
         SPDIAG = SINHIN*(SINHS-DELS)
         TEMP(I) = DIAGIN*SPDIAG
         IF(.NOT.PER) GO TO 201
            Q=Q-T2(I)*P
            R=R-YP(I)*P
 201     CONTINUE
         DX1 = DX2
         DIAG1 = DIAG2
  102 CONTINUE
  103 DIAGIN = 1./(DIAG1-SPDIAG*TEMP(NM1))
      YP(NN) = DIAGIN*(SLPPN-DX2-SPDIAG*YP(NM1))
      IF(.NOT.PER) GO TO 202
         DIAGIN=1./(DIAG1+DIAG11-A2*(P+Q)-SPDIAG*TEMP(NM1))
         YP(NN)=DIAGIN*(DX11-DX1-A2*R-SPDIAG*YP(NM1))
 202  CONTINUE
C
C PERFORM BACK SUBSTITUTION
C
      DO 104 I=2,NN
         IBAK = NP1-I
         YP(IBAK) = YP(IBAK)-TEMP(IBAK)*YP(IBAK+1)
         IF(PER) YP(IBAK)=YP(IBAK)-T2(IBAK)*YP(NN)
  104 CONTINUE
      RETURN
  105 IF (NN .EQ. 2) GO TO 106
C
C IF NO DERIVATIVES ARE GIVEN USE SECOND ORDER POLYNOMIAL
C INTERPOLATION ON INPUT DATA FOR VALUES AT ENDPOINTS.
C
      DELX2 = X(3)-X(2)
      DELX12 = X(3)-X(1)
      C1 = -(DELX12+DELX1)/DELX12/DELX1
      C2 = DELX12/DELX1/DELX2
      C3 = -DELX1/DELX12/DELX2
      SLPP1 = C1*Y(1)+C2*Y(2)+C3*Y(3)
      DELN = X(NN)-X(NM1)
      DELNM1 = X(NM1)-X(NN-2)
      DELNN = X(NN)-X(NN-2)
      C1 = (DELNN+DELN)/DELNN/DELN
      C2 = -DELNN/DELN/DELNM1
      C3 = DELN/DELNN/DELNM1
      SLPPN = C3*Y(NN-2)+C2*Y(NM1)+C1*Y(NN)
      GO TO 101
C
C IF ONLY TWO POINTS AND NO DERIVATIVES ARE GIVEN, USE
C STRAIGHT LINE FOR CURVE
C
  106 YP(1) = 0.
      YP(2) = 0.
      RETURN
      END



      FUNCTION CURV2 (T,N,X,Y,YP,SIGMA,IT)

C
C
C DIMENSION OF           X(N),Y(N),YP(N)
C ARGUMENTS
C
C LATEST REVISION        OCTOBER 22, 1973
C
C PURPOSE                CURV2 INTERPOLATES A CURVE AT A GIVEN POINT
C                        USING A SPLINE UNDER TENSION.  THE SUBROUTINE
C                        CURV1 SHOULD BE CALLED EARLIER TO DETERMINE
C                        CERTAIN NECESSARY PARAMETERS.
C
C ACCESS CARDS           *FORTRAN,S=ULIB,N=CURV
C                        *COSY
C
C USAGE                  YT = CURV2 (T,N,X,Y,YP,SIGMA,IT)
C
C ARGUMENTS
C
C ON INPUT               T
C                          CONTAINS A REAL VALUE TO BE MAPPED ONTO THE
C                          INTERPOLATING CURVE.
C
C                        N
C                          CONTAINS THE NUMBER OF POINTS WHICH WERE
C                          INTERPOLATED TO DETERMINE THE CURVE.
C
C                        X AND Y
C                          ARE ARRAYS CONTAINING THE ABSCISSAE AND
C                          ORDINATES OF THE INTERPOLATED POINTS.
C
C                        YP
C                          IS AN ARRAY WITH VALUES PROPORTIONAL TO THE
C                          SECOND DERIVATIVE OF THE CURVE AT THE NODES.
C
C                        SIGMA
C                          CONTAINS THE TENSION FACTOR (ITS SIGN IS
C                          IGNORED).
C
C                        IT
C                          IS AN INTEGER SWITCH.  IF IT IS NOT 1, THIS
C                          INDICATES THAT THE FUNCTION HAS BEEN CALLED
C                          PREVIOUSLY (WITH N, X, Y, YP AND SIGMA
C                          UNALTERED) AND THAT THIS VALUE OF T EXCEEDS
C                          THE PREVIOUS VALUE.  WITH SUCH INFORMATION
C                          THE FUNCTION IS ABLE TO PERFORM THE
C                          INTERPOLATION MUCH MORE RAPIDLY.  IF A USER
C                          SEEKS TO INTERPOLATE AT A SEQUENCE OF POINTS,
C                          EFFICIENCY IS GAINED BY ORDERING THE VALUES
C                          INCREASING AND SETTING IT TO THE INDEX OF THE
C                          CALL.  IF IT IS 1 THE SEARCH FOR THE INTERVAL
C                          (X(K),X(K+1)) CONTAINING T STARTS WITH K = 1.
C
C                        THE PARAMETERS N, X, Y, YP AND SIGMA SHOULD BE
C                        INPUT UNALTERED FROM THE OUTPUT OF CURV1.
C
C ON OUTPUT              CURV2
C                          CONTAINS THE INTERPOLATED VALUE.  FOR T LESS
C                          THAN X(1), CURV2 = Y(1).  FOR T GREATER THAN
C                          X(N), CURV2 = Y(N).
C
C                        NONE OF THE INPUT PARAMETERS ARE ALTERED.
C
C ENTRY POINTS           CURV2
C
C SPECIAL CONDITIONS     NONE
C
C COMMON BLOCKS          NONE
C
C I/O                    NONE
C
C PRECISION              SINGLE
C
C REQUIRED ULIB          NONE
C ROUTINES
C
C SPECIALIST             RUSSELL K. REW, NCAR, BOULDER, COLORADO  80302
C
C LANGUAGE               FORTRAN
C
C HISTORY                ORIGINALLY WRITTEN BY A. K. CLINE, MARCH, 1972.
C
C
C
C
      INTEGER         N          ,IT
      REAL            T          ,X(N)       ,Y(N)       ,YP(N)      ,
     1                SIGMA
      S = X(N)-X(1)
C
C DENORMALIZE SIGMA
C
      SIGMAP = ABS(SIGMA)*FLOAT(N-1)/S
C
C IF IT.NE.1 START SEARCH WHERE PREVIOUSLY TERMINATED,
C OTHERWISE START FROM BEGINNING
C
      IF (IT .EQ. 1) I1 = 2
C
C SEARCH FOR INTERVAL
C
  101 DO 102 I=I1,N
         IF (X(I)-T) 102,102,103
  102 CONTINUE
      I = N
C
C CHECK TO INSURE CORRECT INTERVAL
C
  103 IF (X(I-1).LE.T .OR. T.LE.X(1)) GO TO 104
C
C RESTART SEARCH  AND RESET I1
C ( INPUT ""IT"" WAS INCORRECT )
C
      I1 = 2
      GO TO 101
C
C SET UP AND PERFORM INTERPOLATION
C
  104 DEL1 = T-X(I-1)
      DEL2 = X(I)-T
      DELS = X(I)-X(I-1)
      EXPS1 = EXP(SIGMAP*DEL1)
      SINHD1 = .5*(EXPS1-1./EXPS1)
      EXPS = EXP(SIGMAP*DEL2)
      SINHD2 = .5*(EXPS-1./EXPS)
      EXPS = EXPS1*EXPS
      SINHS = .5*(EXPS-1./EXPS)
      CURV2 = (YP(I)*SINHD1+YP(I-1)*SINHD2)/SINHS+
     1        ((Y(I)-YP(I))*DEL1+(Y(I-1)-YP(I-1))*DEL2)/DELS
      I1 = I
      RETURN
      END
